from datetime import datetime, timedelta
from typing import Union
from jose import jwt
from fastapi import Cookie, Response, HTTPException
import os

def login_support(response: Response, data: dict, 
                  expires_delta: Union[timedelta, None] = None
                  ):
    
    secret_key = os.environ['SECRET_KEY']
    algorithm = os.environ.get('ALGORITHM', 'HS256')
    secure = os.environ.get('secure', True)

    # algorithm:str="HS256", 
    to_encode = data.copy()
    if expires_delta:
        expire = (datetime.utcnow() + expires_delta).strftime('%s')
    else:
        expire = (datetime.utcnow() + timedelta(minutes=24*60*60)).strftime('%s')
    to_encode.update({"exp": expire})
    encoded_jwt = jwt.encode(to_encode, secret_key, algorithm=algorithm)
    response.set_cookie("auth_token", encoded_jwt, expires=expire, 
                        samesite='strict', secure=secure )
    return

async def auth_required(auth_token:str =  Cookie(None)):
    secret_key = os.environ['SECRET_KEY']

    algorithm = os.environ.get('ALGORITHM', 'HS256')

    x = HTTPException( status_code=401, detail="UnAuthorized")
    if not auth_token:
        print("User is Not logged in")
        raise x
    try:
        jw = jwt.decode(auth_token,key=secret_key, algorithms=algorithm)
        if datetime.fromtimestamp(int(jw.get("exp"))) < datetime.utcnow():
            print("Token Expired")
            raise x

    except Exception as e:
        print("cant parsed JWT")
        print(e)
        raise x



import json
from enum import Enum


class JsonEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, Enum):
            return obj.value
        return json.JSONEncoder.default(self, obj)





import click
from zen_dash.cli import project_management as pm


@click.command()
@click.argument("location")
def create_zen(location):
    if location is None:
        location = "."
    pm.create_zen(location)



    


from zen_dash.objects.flex_data import FlexData
import warnings

warnings.warn("""
from zen_dash import flex_data is depricated, Please use
from zen_dash.objects import flex_data
""", DeprecationWarning)


from enum import Enum
from typing import Optional, List

from zen_dash.support import BaseUpdate


class Rel(Enum):
  ALTERNATE = "alternate"
  AUTHOR =  "author"
  DNS_PREFETCH = "dns-prefetch"
  HELP = "help"
  ICON = "ocpm"
  LICENSE = "license"
  NEXT = "next"
  PINGBACK = "pingback"
  PRECONNECT = "preconnect"
  PREFETCH = "prefetch"
  PRELOAD = "preload"
  PRERENDER = "prerender"
  PREV = "prev"
  SEARCH = "search"
  STYLESHEET = "stylesheet"


class Style(Enum):
  JS= "javascript"
  LINK="link"
  
class CustomScript(BaseUpdate):
  url: Optional[str]
  text: Optional[str]
  type: Style
  rel: Optional[Rel] # Only for link
  custom: Optional[str]
  aysnc: bool = False
  defer: bool = True


class CustomScripts(BaseUpdate):
  scripts: List[CustomScript]



from pathlib import Path
import pathlib
from venv import create
import subprocess

create_project_code = """
from fastapi import FastAPI, Request, Response
from fastapi.responses import HTMLResponse
from fastapi.staticfiles import StaticFiles
from fastapi.templating import Jinja2Templates
import pkg_resources
from zen_dash import sidebar as s
from zen_dash import page as p
from zen_dash import scripts as sc
from pydantic import BaseConfig
from fastapi.middleware.gzip import GZipMiddleware
from app import global_filter as gf
from app import page 

BaseConfig.arbitrary_types_allowed = True  # change #1

app = FastAPI()
app.include_router(gf.router)
app.include_router(page.router)

app.add_middleware(GZipMiddleware, minimum_size=1000)
folder = pkg_resources.resource_filename('zen_dash', 'static/')
templates = Jinja2Templates(directory=folder)


@app.get("/", response_class=HTMLResponse)
async def root(request: Request, response: Response):
    return templates.TemplateResponse("index.html", {"request": request, "test": "test"})


@app.get("/backend/title")
async def title():
    return 'Zen Dash Dashboard'



@app.get("/backend/sidebar", response_model=s.Sidebar)
async def sidebar():
    return s.Sidebar(tabs=[ s.SidebarTab(label="First Page", icon='home') ],
                     filters=[ s.FilterInfo(url=gf.SingleFilter.full_url()) ]
                     )


@app.get("/backend/page_detail", response_model=p.Page)
async def page_detail(fragment: str):
    return p.RenderPage([page.BOXPAGE], fragment)



app.mount("/", StaticFiles(directory=folder), name="static")

"""

page_data = """
from fastapi import APIRouter
from zen_dash import instances as i
from zen_dash import Zen
from zen_dash import ZenPage, page as p

prefix="/backend/page"


class FirstBox(Zen):
    @staticmethod
    def full_url() -> str:
        return f"{prefix}/first_box"

    @staticmethod
    def url() -> str:
        return "/first_box"

    @staticmethod
    async def view():
        import random
        name = random.choice(["Users", "Spent"])
        Value = random.choice(["5009", "200"])
        return i.ReturnData(type=i.InstanceType.BOX,
                            box_data=i.BoxData(
                                icon="person", 
                                name=name, 
                                value=Value),
                            footer="5% increase compare to last week ",
                            tooltip_data=i.ToolTipData(
                                label="my label", disable=False),
                            )

router = APIRouter(
    prefix=prefix,
    tags=["page"],
    responses={404: {"description": "Not found"}},
)

@router.post(FirstBox.url(), response_model=i.ReturnData)
async def d3():
    return await FirstBox.view()
                            

box_page = p.Page(
        rows=[
            p.Row(data=[  
                p.Instance(url=FirstBox.full_url())
            ])
        ])


BOXPAGE = ZenPage(
    name= "BoxPage",
    icon= "home",
    page= box_page,
    tab_number=0
)
    """

create_global_filter = """
from fastapi import APIRouter
from zen_dash import instances as i
from zen_dash import Zen

prefix="/backend/global_filters"
router = APIRouter(
    prefix=prefix,
    tags=["filters"],
    responses={404: {"description": "Not found"}},
)



class SingleFilter(Zen):
    @staticmethod
    def full_url() -> str:
        return f"{prefix}/single_filter"

    @staticmethod
    def url() -> str:
        return f"/single_filter"

    @staticmethod
    def view():
        s = i.ReturnData(
            title="Simple Filter",
            type=i.InstanceType.SIMPLE_FILTER,
            simple_filter_data=i.SimpleFilterData(
                name="simple_filter",
                data=["Test 1", "My 2"],
                selected=['Test 1'])
        )

        return s

        
@router.post(SingleFilter.url(), response_model=i.ReturnData)
async def d3():
    return SingleFilter.view()
                            

"""

qa_sh = """
uvicorn main:app --reload --reload-dir .
"""


prod_sh = """
uvicorn main:app --workers 10
"""


requirements ="""
zen_dash
uvicorn
"""



def create_zen(location):
    if location == ".":
        build_location = pathlib.Path()
    elif location[0] == "/":
        build_location = pathlib.Path(location)
    else:
        build_location = pathlib.Path().joinpath(location)
    
    Path(build_location.joinpath("app").joinpath("global_filter")).mkdir(parents=True, exist_ok=True)
    Path(build_location.joinpath("app").joinpath("page")).mkdir(parents=True, exist_ok=True)
    Path(build_location.joinpath(".venv")).mkdir(parents=True, exist_ok=True)

    main_file_location = build_location.joinpath("main.py").absolute()

    with open(main_file_location, "w") as f:
        f.write(create_project_code)

    init_file = build_location.joinpath("app").joinpath("global_filter").joinpath("__init__.py")
    with open(init_file, "w") as f:
        f.write(create_global_filter)

    global_file = build_location.joinpath("app").joinpath("page").joinpath("__init__.py")
    with open(global_file, "w") as f:
        f.write(page_data)
    
    qa_file = build_location.joinpath("qa.sh")
    with open(qa_file, "w") as f:
        f.write(qa_sh)
    
    prod_file = build_location.joinpath("prod.sh")
    with open(prod_file, "w") as f:
        f.write(prod_sh)

    requirement_file = build_location.joinpath("requirements.txt")
    with open(requirement_file, "w") as f:
        f.write(requirements)

    venv_file = str(build_location.joinpath(".venv").absolute())
    create(venv_file, with_pip=True)
    

    requirement_file = str(requirement_file.absolute())
    pip_location = str(build_location.joinpath(".venv").joinpath("bin").joinpath("pip3").absolute())
    subprocess.run([pip_location, "install", "-r", requirement_file], capture_output=True)






from typing import Optional, List, Union
from zen_dash.objects import ZenPage

from zen_dash.support import BaseUpdate
from importlib.metadata import version
from pydantic import validator

class SidebarTab(BaseUpdate):
    label: str
    icon: Optional[str] = 'Home'

class SidebarGroup(BaseUpdate):
    name: str
    subtabs: List[SidebarTab]

class FilterInfo(BaseUpdate):
    url: str

class Sidebar(BaseUpdate):
   
    tabs: List[Union[SidebarTab, SidebarGroup]]
    filters: List[FilterInfo]
    size: str = '300px'
    library_version: str = version("zen-dash")

    @validator('tabs')
    def first_cant_be_group(cls, v):
        if len(v)> 1:
            if not isinstance(v[0], SidebarTab):
                raise Exception("First tab can't be SidebarGroup")
        return v


class GroupSidebarList(BaseUpdate):
    name: str
    sub_pages: List[ZenPage]

class SidebarList(BaseUpdate):
    pages: List[ZenPage]
    groupSidebar: List[GroupSidebarList]


def get_page_dict(pages: List[ZenPage]):
    page_dict = {}
    for p in pages:
        if p.tab_number is None:
            raise Exception("Tab number is not define in Zen Page")
        
        page_number = p.tab_number
        if p.subtab_number is None:
            
            page_dict[page_number] = p
        else:
            if page_dict.get(page_number) is None:
                page_dict[page_number] = {}
            elif isinstance(page_dict.get(page_number), dict):
                pass
            elif not isinstance(page_dict.get(page_number), dict):
                raise Exception(f"Please Check tab_number and subtab_number")
            
            sub_number = p.subtab_number
            page_dict[page_number][sub_number] = p
    return page_dict

def RenderTabs(sidebar_list: SidebarList):
    len_ = len(sidebar_list.pages)
    len_ += len(sidebar_list.groupSidebar)
    
    tabs = [None for i in range(len_)]
    
    for p in sidebar_list.pages:
        page_index = p.tab_number
        tabs[page_index] = SidebarTab(label=p.name, icon=p.icon)
    
    for subtab in sidebar_list.groupSidebar:
        sub_tab = [None for i in range(len(subtab.sub_pages))] 
        for sub_page in subtab.sub_pages:
            if sub_page.subtab_number is None:
                raise Exception(f"Please provde subpage for {sub_page.name}")
            sub_page_index = sub_page.subtab_number
            sub_tab[sub_page_index] = SidebarTab(label=sub_page.name, icon=sub_page.icon)
        page_index = sub_page.tab_number
        tabs[page_index] = SidebarGroup(name= subtab.name, subtabs=sub_tab)

    for i in tabs:
        if isinstance(i, SidebarTab):
            pass
        elif isinstance(i, SidebarGroup):
            i: SidebarGroup
            for index, j in enumerate(i.subtabs):
                if j is None:
                    raise Exception(f"{i.name} tab doesn't have define subtab_number {index}")
        else:
            raise Exception("Missing Tab")
    return tabs







    


from enum import Enum, unique
import json
from json import JSONEncoder
from pydantic import BaseConfig, ValidationError, root_validator, validator
from typing import List, Optional, Dict, Union
from zen_dash.support.encoder import JsonEncoder
from zen_dash.objects.flex_data import FlexData
from zen_dash.support import BaseUpdate

BaseConfig.arbitrary_types_allowed = True  # change #1

class InstanceType(Enum):
    """ Docstring for class InstanceType
    Instance Type required for ReturnData

    :return: Instance Type for Enum
    :rtype: Enum

    """
    DATE= "date"
    BOX = "box"
    TABLE = "table"
    CHART = "chart"
    CHECKBOX = "checkbox"
    RADIO = "radio"
    SIMPLE_FILTER = "simple_filter"
    SIMPLE_SERVER_FILTER = "simple_server_filter"
    GROUP_FILTER = "group_filter"
    SLIDER = "slider"
    BUTTON_TOGGLE = "button_toggle"
    TOGGLE = "toggle"
    MULTI_LIST = "multi_list"
    MULTI_TABS = "multi_tabs"
    MULTI_EXPAND = "multi_expand"
    INPUT = "input"
    DOWNLOAD = "download"
    UPLOAD = "upload"
    IMAGE = "image"
    HIGHCHART = "highchart"
    IFRAME = "iframe"
    CUSTOM_HTML="custom_html"
    BUTTON = "button"
    FORM = "form"
    FLOATING_BUTTON = "floating_button"

class BoxData(BaseUpdate):
    icon: str
    name: str
    value: str
    

class DateTimeData(BaseUpdate):
    name: str
    first_date: str
    second_date: Optional[str]

class TableColumn(BaseUpdate):
    columnDef: str
    header: str

class TableData(BaseUpdate):
    columns: List[TableColumn]
    data: List[Dict]
    name: str
    items_per_page_options: List[int] = [5, 10, 20]
    items_per_page: int = 5
    allow_download: bool = True
    
class ChartData(BaseUpdate):
    name: str
    data: Dict

class CheckBoxInstance(BaseUpdate):
    name: str
    selected: bool
    
class Style(Enum):
    horizontal = "Horizontal"
    vertical = "Vertical"

class CheckBoxData(BaseUpdate):
    data: List[CheckBoxInstance]
    style: Style = Style.horizontal
    name: str

class RadioData(BaseUpdate):
    data: List[str]
    style: Style = Style.horizontal
    name: str
    selected: Optional[str] = ""

class SliderData(BaseUpdate):
    name: str
    max: float
    min: float
    step: float
    thumbLabel: bool = True
    invert: bool = False
    vertical: bool = False
    value: Optional[float]

class ButtonToggleInstance(BaseUpdate):
    name: str
    selected: bool = False

class ButtonToggleData(BaseUpdate):
    name: str
    multi: bool = False
    data: List[ButtonToggleInstance]


class GroupedFilterDataInstance(BaseUpdate):
    group_name: str
    group_data: List[str]

class InputData(BaseUpdate):
    label: Optional[str]
    name: str
    value: Optional[str]


class GroupedFilterData(BaseUpdate):
    multi: bool = False
    name: str
    url: Optional[str]
    data: List[GroupedFilterDataInstance]
    selected: List[str] = []


class SimpleFilterData(BaseUpdate):
    multi: bool = False
    name: str
    data: List[str]
    selected: List[str] = []

class SimpleServerSideFilterData(SimpleFilterData):
    url: Optional[str]


class ToggleData(BaseUpdate):
    name: str
    checked: bool = False


class MultiURLInfo(BaseUpdate):
    name: Optional[str]
    with_card: bool = False
    url: str

class MultiData(BaseUpdate):
    urls: List[MultiURLInfo]


class ReactiveData(BaseUpdate):
    full_rective: Optional[bool] = False
    reactive_ids: Optional[List[str]] = []
    hidden: bool = False

class DownloadData(BaseUpdate):
    label: str
    url: str
    name: str

class ImageData(BaseUpdate):
    url: str
    height: str
    width: str

class UploadData(BaseUpdate):
    url: str
    multi: bool=  False
    name: str

class HighChartType(Enum):
    CHART = "chart"
    STOCK = "stock"
    MAP = "map"

class HighChartData(BaseUpdate):
    type: HighChartType = HighChartType.CHART
    config: Dict

class IframeData(BaseUpdate):
    url: str
    width: str = "1100"
    height: str = "400"

class ToolTipData(BaseUpdate):
    label: str = ''

class DialogBox(BaseUpdate):
    url: str = ''
    height: str = '60%'
    width: str = '60%'

class CustomHTML(BaseUpdate):
    name: str
    html: str
    full_custom: bool = False
    script: Optional[str]

class TargetAttribute(Enum):
    Blank = "_blank"
    PARENT = "_parent"
    SELF = "_self"
    TOP = "_top"

class ButtonData(BaseUpdate):
    url: str
    name: str
    redirect: bool = False
    target_attribute:TargetAttribute = TargetAttribute.Blank

class SubmitFormData(BaseUpdate):
    name: str
    url: str

class FormData(BaseUpdate):
    name: str
    submit_info: SubmitFormData
    form_schema: Dict
    ui_schema: Dict
    data: Dict


class FABStyle(Enum):
    FAB= "fab"
    MINIFAB = "mini_fab"
    ICON = "icon"

class FABColor(Enum):
    PRIMARY = "primary"
    ACCENT = "accent"
    WARN = "warn"


class ButtonFloating(BaseUpdate):
    url: str
    name: str
    redirect: bool = False
    style: Dict[str, str]
    icon: str
    fab_style: FABStyle = FABStyle.FAB
    color: FABColor = FABColor.ACCENT
    target_attribute:TargetAttribute = TargetAttribute.PARENT



class ReturnData(BaseUpdate):
    """
    Main return object use for everything except /backend/page_detail, /backend/sidebar, and /backend/title
    depdning upon which type (InstanceType) you select, you have to select corsponding return value 

    for example, if you select BOX as input, you have to choose box_data as one of  input.

    Example: i.ReturnData(type=i.InstanceType.BOX, box_data=i.BoxData(icon="attach_money", name="User Spent", value="$5000"), footer="10% increase compare to last week ")

    :param type: InstanceType this will dectate what type of output will be render
    :param title: Optional[str] tile of mat box in which output will be render
    
    """

    type:InstanceType
    title: Optional[str]
    chart_data: Optional[ChartData]
    box_data: Optional[BoxData]
    date_data: Optional[DateTimeData]
    checkbox_data: Optional[CheckBoxData]
    radio_data: Optional[RadioData]
    slider_data: Optional[SliderData]
    button_toggle_data: Optional[ButtonToggleData]
    table_data: Optional[TableData]
    toggle_data: Optional[ToggleData]
    multi_data: Optional[MultiData]
    simple_filter_data: Optional[SimpleFilterData] 
    simple_server_filter_data: Optional[SimpleServerSideFilterData] 
    group_filter_data: Optional[GroupedFilterData ]
    input_data: Optional[InputData]
    download_data: Optional[DownloadData]
    upload_data: Optional[UploadData]
    image_data: Optional[ImageData]
    highchart_data: Optional[HighChartData]
    iframe_data: Optional[IframeData]
    custom_html_data: Optional[CustomHTML]
    footer: Optional[str]
    flex: Optional[FlexData] = FlexData()
    reactive: Optional[ReactiveData] = ReactiveData()
    tooltip_data: Optional[ToolTipData]
    dialog_data: Optional[DialogBox]
    button_data: Optional[ButtonData]
    form_data: Optional[FormData]
    floating_button_data: Optional[ButtonFloating]

    @root_validator
    def validator_type_match(cls, field_values):
        if (field_values["type"] == InstanceType.BOX) and (field_values["box_data"] is None):
            raise ValueError("You have selected InstanceType.BOX, and box_data is missing")
        elif (field_values["type"] == InstanceType.DATE) and (field_values["date_data"] is None):
            raise ValueError("You have selected InstanceType.DATE, and date_data is missing")
        elif (field_values["type"] == InstanceType.TABLE) and (field_values["table_data"] is None):
            raise ValueError("You have selected InstanceType.TABLE, and table_data is missing")
        elif (field_values["type"] == InstanceType.CHART) and (field_values["chart_data"] is None):
            raise ValueError("You have selected InstanceType.CHART, and chart_data is missing")
        elif (field_values["type"] == InstanceType.CHECKBOX) and (field_values["checkbox_data"] is None):
            raise ValueError("You have selected InstanceType.CHECKBOX, and checkbox_data is missing")
        elif (field_values["type"] == InstanceType.RADIO) and (field_values["radio_data"] is None):
            raise ValueError("You have selected InstanceType.RADIO, and radio_data is missing")
        elif (field_values["type"] == InstanceType.SIMPLE_FILTER) and (field_values["simple_filter_data"] is None):
            raise ValueError("You have selected InstanceType.SIMPLE_FILTER, and simple_filter_data is missing")
        elif (field_values["type"] == InstanceType.SIMPLE_SERVER_FILTER) and (field_values["simple_server_filter_data"] is None):
            raise ValueError("You have selected InstanceType.SIMPLE_SERVER_FILTER, and simple_server_filter_data is missing")
        elif (field_values["type"] == InstanceType.GROUP_FILTER) and (field_values["group_filter_data"] is None):
            raise ValueError("You have selected InstanceType.GROUP_FILTER, and group_filter_data is missing")
        elif (field_values["type"] == InstanceType.SLIDER) and (field_values["slider_data"] is None):
            raise ValueError("You have selected InstanceType.SLIDER, and slider_data is missing")
        elif (field_values["type"] == InstanceType.BUTTON_TOGGLE) and (field_values["button_toggle_data"] is None):
            raise ValueError("You have selected InstanceType.BUTTON_TOGGLE, and button_toggle_data is missing")
        elif (field_values["type"] == InstanceType.TOGGLE) and (field_values["toggle_data"] is None):
            raise ValueError("You have selected InstanceType.TOGGLE, and toggle_data is missing")
        elif (field_values["type"] == InstanceType.MULTI_LIST) and (field_values["multi_data"] is None):
            raise ValueError("You have selected InstanceType.MULTI_LIST, and multi_data is missing")
        elif (field_values["type"] == InstanceType.MULTI_TABS) and (field_values["multi_data"] is None):
            raise ValueError("You have selected InstanceType.MULTI_TABS, and multi_data is missing")
        elif (field_values["type"] == InstanceType.MULTI_EXPAND) and (field_values["multi_data"] is None):
            raise ValueError("You have selected InstanceType.MULTI_EXPAND, and multi_data is missing")
        elif (field_values["type"] == InstanceType.INPUT) and (field_values["input_data"] is None):
            raise ValueError("You have selected InstanceType.INPUT, and input_data is missing")
        elif (field_values["type"] == InstanceType.DOWNLOAD) and (field_values["download_data"] is None):
            raise ValueError("You have selected InstanceType.DOWNLOAD, and download_data is missing")
        elif (field_values["type"] == InstanceType.UPLOAD) and (field_values["upload_data"] is None):
            raise ValueError("You have selected InstanceType.UPLOAD, and upload_data is missing")
        elif (field_values["type"] == InstanceType.IMAGE) and (field_values["image_data"] is None):
            raise ValueError("You have selected InstanceType.IMAGE, and image_data is missing")
        elif (field_values["type"] == InstanceType.IFRAME) and (field_values["iframe_data"] is None):
            raise ValueError("You have selected InstanceType.IFRAME, and iframe_data is missing")
        elif (field_values["type"] == InstanceType.HIGHCHART) and (field_values["highchart_data"] is None):
            raise ValueError("You have selected InstanceType.HIGHCHART, and highchart_data is missing")
        elif (field_values["type"] == InstanceType.CUSTOM_HTML) and (field_values["custom_html_data"] is None):
            raise ValueError("You have selected InstanceType.CUSTOM_HTML, and custom_html_data is missing")
        elif (field_values["type"] == InstanceType.BUTTON) and (field_values["button_data"] is None):
            raise ValueError("You have selected InstanceType.BUTTON, and button_data is missing")
        elif (field_values["type"] == InstanceType.FORM) and (field_values["form_data"] is None):
            raise ValueError("You have selected InstanceType.FORM, and form_data is missing")
        elif (field_values["type"] == InstanceType.FLOATING_BUTTON) and (field_values["floating_button_data"] is None):
            raise ValueError("You have selected InstanceType.FLOATING_BUTTON, and floating_button_data is missing")
        elif (field_values["type"] == InstanceType.FLOATING_BUTTON) and (field_values["footer"] is not None):
            raise ValueError("footer is not supported with InstanceType.FLOATING_BUTTON")
        elif (field_values["type"] == InstanceType.FLOATING_BUTTON) and (field_values["tooltip_data"] is not None):
            raise ValueError("tooltip_data is not supported with InstanceType.FLOATING_BUTTON")
        elif (field_values["type"] == InstanceType.FLOATING_BUTTON) and (field_values["dialog_data"] is not None):
            raise ValueError("dialog_data is not supported with InstanceType.FLOATING_BUTTON")

        return field_values

@unique
class UpdateInstanceType(str, Enum):
    """ Docstring for class UpdateInstanceType
    Instance Type required for UpdateReturnData

    :return: Update Instance Type for Enum
    :rtype: Enum

    """
    SIMPLE_FILTER = "simple_filter"
    BOX = "box"
    FORM = "form"
    BUTTON_RESULT = "button_result"


class DisplayStatus(Enum):
    WARN= "warn"
    SUCCESS = "success"
    ERROR = "error"


class Display(BaseUpdate):
    duration: int = 5
    message: Optional[str ]
    status: DisplayStatus = DisplayStatus.SUCCESS

class DisplayDialog(BaseUpdate):
      height: str = '500px'
      width: str = '500px'
      custom_message: str

class UpdateReturnData(BaseUpdate):
    type: UpdateInstanceType
    simple_fitler_data: Optional[List[str]]
    box_data: Optional[BoxData]
    display: Optional[Display]
    display_dialog: Optional[DisplayDialog]

    @root_validator
    def validator_type_match(cls, field_values):

        if (field_values["type"] == UpdateInstanceType.BOX) :
            if (field_values["box_data"] is None):
                raise ValueError("You have selected UpdateInstanceType.BOX, and box_data is missing")
            if (field_values["display_dialog"] is not None):
                raise ValueError("You have selected UpdateInstanceType.BOX, and display_dialog is not supported")
        elif (field_values["type"] == UpdateInstanceType.SIMPLE_FILTER) :
            if (field_values["simple_fitler_data"] is None):
                raise ValueError("You have selected UpdateInstanceType.SIMPLE_FILTER, and simple_fitler_data is missing")
            if (field_values["display_dialog"] is not None):
                raise ValueError("You have selected UpdateInstanceType.SIMPLE_FILTER, and display_dialog is not supported")

        elif (field_values["type"] == UpdateInstanceType.BUTTON_RESULT) and (field_values["display"] is None):
            raise ValueError("You have selected UpdateInstanceType.BUTTON_RESULT, and display is missing")

        if (field_values["type"] == UpdateInstanceType.FORM):
            if (field_values["display"] is None):
                raise ValueError("You have selected UpdateInstanceType.FORM, and display is missing")
        
        return field_values

    def json(self):
        d = self.dict()
        return json.dumps(d, cls=JsonEncoder)



from typing import List, Union
from uuid import uuid4

class TAG:
    def __init__(self, tag:str, childern: List[any], id:str=None, style:Union[dict, str]='', other_attributes:dict= {}):
        self.id = id
        self.style = style
        self.tag = tag
        self.childern = childern
        self.other_attributes = other_attributes

    @property
    def get_id(self):
        if self.id is None:
            return ''        
        else:
            return f"id='{self.id}'"

    def compile(self):
       
        if not self.style:
            self.style = ""
        elif isinstance(self.style, str):
            pass
        elif isinstance(self.style, dict):
            temp = ''
            for key, value in self.style.items():
                t = f'{key}:{value};'
                temp = temp + ' ' + t
            self.style = temp
        else:
            raise Exception('style is not right')
        if not self.childern:
            raise Exception('Data is not provided')
        
        others  = ''
        for key, value in self.other_attributes.items():
            others += f' {key}="{value}"'
        if isinstance(self.childern, str):
            return f'<{self.tag} {self.get_id} style="{self.style}" {others} >{self.childern}</{self.tag}>'
        if isinstance(self.childern, TAG):
           
            return f'<{self.tag} {self.get_id} style="{self.style}" {others} >{self.childern.compile()}</{self.tag}>'
        elif isinstance(self.childern, list):
            d = [ ]
            for dd in self.childern:
                if isinstance(dd, str):
                    d.append(dd)
                elif isinstance(dd, TAG):
                    d.append(dd.compile())
                else:
                    raise Exception('data need to be str or TAG type')
            d = ' '.join(d)
            t = ''
            for key, value in self.other_attributes.items():
                t += f' {key}="{value}"'
            return f'<{self.tag} {self.get_id} style="{self.style}" {others}>{d}</{self.tag}>'
        else:
            raise Exception('data need to be str or TAG type')

class A(TAG):
    def __init__(self, childern: List[any], id:str=None, style:Union[dict, str]='', other_attributes:dict= {}):
        super().__init__('a', childern=childern, id=id, style=style, other_attributes=other_attributes )

class SPAN(TAG):
    def __init__(self, childern: List[any], id:str=None, style:Union[dict, str]='', other_attributes:dict= {}):
        super().__init__('span', childern=childern, id=id, style=style, other_attributes=other_attributes )

class H1(TAG):
    def __init__(self, childern: List[any], id:str=None, style:Union[dict, str]='', other_attributes:dict= {}):
        super().__init__('h1', childern=childern, id=id, style=style, other_attributes=other_attributes )

class H2(TAG):
    def __init__(self, childern: List[any], id:str=None, style:Union[dict, str]='', other_attributes:dict= {}):
        super().__init__('h2', childern=childern, id=id, style=style, other_attributes=other_attributes )

class H3(TAG):
    def __init__(self, childern: List[any], id:str=None, style:Union[dict, str]='', other_attributes:dict= {}):
        super().__init__('h3', childern=childern, id=id, style=style, other_attributes=other_attributes )

class H4(TAG):
    def __init__(self, childern: List[any], id:str=None, style:Union[dict, str]='', other_attributes:dict= {}):
        super().__init__('h4', childern=childern, id=id, style=style, other_attributes=other_attributes )

class H5(TAG):
    def __init__(self, childern: List[any], id:str=None, style:Union[dict, str]='', other_attributes:dict= {}):
        super().__init__('h5', childern=childern, id=id, style=style, other_attributes=other_attributes )

class H6(TAG):
    def __init__(self, childern: List[any], id:str=None, style:Union[dict, str]='', other_attributes:dict= {}):
        super().__init__('h6', childern=childern, id=id, style=style, other_attributes=other_attributes )

class DIV(TAG):
    def __init__(self, childern: List[any], id:str=None, style:Union[dict, str]='', other_attributes:dict= {}):
        super().__init__('div', childern=childern, id=id, style=style, other_attributes=other_attributes )

class TITLE(TAG):
    def __init__(self, childern: List[any], id:str=None, style:Union[dict, str]='', other_attributes:dict= {}):
        super().__init__('title', childern=childern, id=id, style=style, other_attributes=other_attributes )

class P(TAG):
    def __init__(self, childern: List[any], id:str=None, style:Union[dict, str]='', other_attributes:dict= {}):
        super().__init__('p', childern=childern, id=id, style=style, other_attributes=other_attributes )

class B(TAG):
    def __init__(self, childern: List[any], id:str=None, style:Union[dict, str]='', other_attributes:dict= {}):
        super().__init__('b', childern=childern, id=id, style=style, other_attributes=other_attributes )

class I(TAG):
    def __init__(self, childern: List[any], id:str=None, style:Union[dict, str]='', other_attributes:dict= {}):
        super().__init__('i', childern=childern, id=id, style=style, other_attributes=other_attributes )

class U(TAG):
    def __init__(self, childern: List[any], id:str=None, style:Union[dict, str]='', other_attributes:dict= {}):
        super().__init__('u', childern=childern, id=id, style=style, other_attributes=other_attributes )

class CENTER(TAG):
    def __init__(self, childern: List[any], id:str=None, style:Union[dict, str]='', other_attributes:dict= {}):
        super().__init__('center', childern=childern, id=id, style=style, other_attributes=other_attributes )

class EM(TAG):
    def __init__(self, childern: List[any], id:str=None, style:Union[dict, str]='', other_attributes:dict= {}):
        super().__init__('em', childern=childern, id=id, style=style, other_attributes=other_attributes )

class SUP(TAG):
    def __init__(self, childern: List[any], id:str=None, style:Union[dict, str]='', other_attributes:dict= {}):
        super().__init__('sup', childern=childern, id=id, style=style, other_attributes=other_attributes )

class SUB(TAG):
    def __init__(self, childern: List[any], id:str=None, style:Union[dict, str]='', other_attributes:dict= {}):
        super().__init__('sub', childern=childern, id=id, style=style, other_attributes=other_attributes )

class OL(TAG):
    def __init__(self, childern: List[any], id:str=None, style:Union[dict, str]='', other_attributes:dict= {}):
        super().__init__('ol', childern=childern, id=id, style=style, other_attributes=other_attributes )

class LI(TAG):
    def __init__(self, childern: List[any], id:str=None, style:Union[dict, str]='', other_attributes:dict= {}):
        super().__init__('li', childern=childern, id=id, style=style, other_attributes=other_attributes )

class UL(TAG):
    def __init__(self, childern: List[any], id:str=None, style:Union[dict, str]='', other_attributes:dict= {}):
        super().__init__('ul', childern=childern, id=id, style=style, other_attributes=other_attributes )



from enum import Enum
from typing import List, Optional

from zen_dash.support import BaseUpdate
from typing import Dict

class Instance(BaseUpdate):
    url: str

class Row(BaseUpdate):
    data: List[Instance]
    layoutGap: Optional[str] = "30px"

class FABURL(BaseUpdate):
    url: str

class Page(BaseUpdate):
    rows: List[Row]
    floating_button_url: List[FABURL] = []
